@@ -4,7 +4,7 @@
 
     Test the sphinx.quickstart module.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,9 +15,7 @@ import pytest
 
 from sphinx import application
 from sphinx.cmd import quickstart as qs
-from sphinx.util.console import nocolor, coloron
-from sphinx.util.pycompat import execfile_
-
+from sphinx.util.console import coloron, nocolor
 
 warnfile = StringIO()
 
@@ -109,7 +107,7 @@ def test_quickstart_defaults(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    execfile_(conffile, ns)
+    exec(conffile.read_text(), ns)
     assert ns['extensions'] == []
     assert ns['templates_path'] == ['_templates']
     assert ns['project'] == 'Sphinx Test'
@@ -159,13 +157,13 @@ def test_quickstart_all_answers(tempdir):
     conffile = tempdir / 'source' / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    execfile_(conffile, ns)
+    exec(conffile.read_text(), ns)
     assert ns['extensions'] == [
         'sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.todo'
     ]
     assert ns['templates_path'] == ['.templates']
     assert ns['source_suffix'] == '.txt'
-    assert ns['master_doc'] == 'contents'
+    assert ns['root_doc'] == 'contents'
     assert ns['project'] == 'STASI™'
     assert ns['copyright'] == '%s, Wolfgang Schäuble & G\'Beckstein' % \
         time.strftime('%Y')
@@ -193,7 +191,7 @@ def test_generated_files_eol(tempdir):
     qs.generate(d)
 
     def assert_eol(filename, eol):
-        content = filename.bytes().decode()
+        content = filename.read_bytes().decode()
         assert all([l[-len(eol):] == eol for l in content.splitlines(True)])
 
     assert_eol(tempdir / 'make.bat', '\r\n')
@@ -240,7 +238,7 @@ def test_default_filename(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    execfile_(conffile, ns)
+    exec(conffile.read_text(), ns)
 
 
 def test_extensions(tempdir):
@@ -250,5 +248,5 @@ def test_extensions(tempdir):
     conffile = tempdir / 'conf.py'
     assert conffile.isfile()
     ns = {}
-    execfile_(conffile, ns)
+    exec(conffile.read_text(), ns)
     assert ns['extensions'] == ['foo', 'bar', 'baz']
